package bank3tier._lgTest; //UsingLgExample.java

/*
 * 2002-04-30  Knabe  Umstellung auf Verwendung von Settern statt parametrierter
 *                    erfassen-Operationen, Zugriff auf Lg-Objekte nur ber
 *                    deren Interfaces, bersicht ber Kontostnde aufgetrennt:
 *                    Ausgabe hier, Zusammenstellung in LgKundeImpl.
 * 2001-11-19  Knabe  Umgestellt auf lokales LgDatum statt java.sql.Date
 * Created on 28. Februar 2001, 15:50
 */


import bank3tier.lg.LgApplication;
import bank3tier.lg.LgKunde;
import bank3tier.lg.LgKunde_List;
import bank3tier.lg.LgKonto;
import bank3tier.lg.LgKontoStand;
import bank3tier.lg.LgKontoStand_List;
import bank3tier.lg.LgDatum;

/**
  Zweck: Erzeugt einige Kunden, einige von diesen verwaltete Konten
  und fhrt einige berweisungen aus.

  @author  Christoph Knabe
 */
public class UsingLgExample extends Object {

    /** Do not create objects of me from outside*/
    private UsingLgExample() {
    }//UsingLgExample()

    /**create objects, link and work with them*/
    public void work() throws Exception {
        LgApplication.init();

        //Einige Kunden erfassen:
        final LgKunde mller = LgApplication.newKunde();
        mller.setName("Manfred Mller");
        mller.setGeburt(LgDatum.fromString("27.02.1960"));
        mller.erfassen();
        final LgKunde knabe = LgApplication.newKunde();
        knabe.setName("Christoph Knabe");
        knabe.setGeburt(LgDatum.fromString("31.05.1954"));
        knabe.erfassen();
        final LgKunde skoric = LgApplication.newKunde();
        skoric.setName("Gordana Skoric-Knabe");
        skoric.setGeburt(LgDatum.fromString("14.11.1962"));
        skoric.erfassen();

        //Fr diese Kunden Konten erffnen:
        final LgKonto mllerGiroKonto = mller.kontoErffnen();
        final LgKonto mllerSparKonto = mller.kontoErffnen();
        final LgKonto knabeKonto  = knabe.kontoErffnen();
        final LgKonto TSV_GuthsmuthsKonto = knabe.kontoErffnen();

        //Skoric als Verwalter fr knabeKonto einsetzen:
        knabeKonto.verwalterHinzufgen(skoric);
        //Mller als Verwalter fr Konto des TSV Guthsmuths einsetzen:
        TSV_GuthsmuthsKonto.verwalterHinzufgen(mller);

        //Einige berweisungen ttigen:
        mllerGiroKonto.berweisen(mllerSparKonto, 200000, mller);
        mllerGiroKonto.berweisen(TSV_GuthsmuthsKonto, 2000, mller);
        knabeKonto.berweisen(mllerGiroKonto, 1998, knabe);

        //Zeige Liste aller Kunden:
        final LgKunde_List kundeList = LgApplication.dummyKunde().alleSuchen();
        kundenlisteDrucken(kundeList);

        //Zeige Status und Kontostand aller von Kunde 0 verwalteten Konten:
        final LgKunde kunde0 = kundeList.get(0);
        final LgKontoStand_List kontoStaende = kunde0.getKontoStandList();
        System.out.println("-----------------------------");
        System.out.println("KontenStnde von Kunde " + kunde0.getName());
        System.out.println("-----------------------------");
        System.out.println("KontoOid,\tStatus,\tSaldo");
        for(final LgKontoStand_List.Iterator it=kontoStaende.listIterator();it.hasNext();){
            final LgKontoStand kontoStand = it.next();
            final String status
            = kontoStand.getInhaber() ? "Inhaber" : "Verwalter";
            System.out.println("#" + kontoStand.getOid() + " " + status
                + " " + kontoStand.getSaldoCt()
            );
        }

        //Demonstriert eindeutiges Suchen:
        final LgDatum suchDatum = LgDatum.fromString("31.05.1954");
        System.out.println("Zeige den Kunden, der am " + suchDatum + " geboren wurde:");
        final LgKunde geb540531Kunde = LgApplication.dummyKunde().suchen(suchDatum);
        System.out.println(geb540531Kunde);

    }//work

    /*package*/ static void kundenlisteDrucken(final LgKunde_List i_kundeList){
        System.out.println("Liste aller Kunden");
        System.out.println("==================");
        System.out.println("OID,\tName,\tGeburt");
        for(LgKunde_List.Iterator i=i_kundeList.listIterator(); i.hasNext();){
          final LgKunde k = i.next();
          System.out.println(k.getOid() + ",\t" + k.getName() + ",\t" + k.getGeburt());
        }
    }

    /**Do some work using the application logic tier (Lg) */
    public static void main (final String[] i_args) throws java.sql.SQLException {
        final UsingLgExample example = new UsingLgExample();
        try{example.work();}
        catch(Exception ex){multex.Msg.printReport(ex);  System.exit(1);}
    }

}//UsingLgExample