/*
 * DbService.java
 *
 * Created on 11. April 2002, 17:18
 */

package bank3tier.db;

/**Contains the static service operations for the database access.
 *
 * @author  administrator
 * @version
 */
public class DbService {


    /**Opens the database*/
    public static void dbOpen(final DbApplication io_dbApplication) throws multex.Failure
    {
        try{
            io_dbApplication.openDB();
            io_dbApplication.getSession().connection.setAutoCommit(false);
            _dbApplication = io_dbApplication;
        }catch(Exception ex){
            throw new multex.Failure("Failure opening database", ex);
        }
    }

    /**Creates and returns a new Table object for managing the access to the table
      for storing objects of the persistent class i_class.
    */
    public static jora.Table dbNewTable(final Class i_persistentClass)
    throws multex.Failure
    {
        String className = null;
        try{
            DbObjectImpl.dbCheckPersistentClass(i_persistentClass);
            className = i_persistentClass.getName();
            final String localClassName
            = className.substring(className.lastIndexOf('.') + 1);
            final StringBuffer buf = new StringBuffer(localClassName);
            final int len = localClassName.length();
            final String postfix = "Impl";
            if(!localClassName.endsWith(postfix)){throw new IllegalArgumentException
              ("Persistent class name " + className + " does not end with '" + postfix);
            }
            if(localClassName.endsWith(postfix)){buf.delete(len-postfix.length(), len);}
            final String prefix = "Lg";
            if(localClassName.startsWith(prefix)){buf.delete(0, prefix.length());}
            final String tableName = buf.toString().toLowerCase();
            return new jora.Table(
                className, tableName, _dbApplication.getSession(), "oid"
            );
        }catch(Exception ex){throw new multex.Failure(
                "Cannot create database table object for persistent class {0}",
                ex, className
            );
        }
    }

    /**Creates the application specific database tables*/
    public static void dbCreateTables() throws multex.Failure {
        try{
            _dbApplication.createTables(DbObject.oidSuffix);
            dbEndTransaction(true);
        }catch(Exception ex){
            throw new multex.Failure("Failure creating the application "
                + "specific database tables", ex
            );
        }
    }

    /**Deletes the application specific database tables*/
    public static void dbDeleteTables() throws multex.Failure {
        try{
            final jora.Session session = _dbApplication.getSession();
            session.closePreparedStatements();
            _dbApplication.deleteTables();
            dbEndTransaction(true);
            session.close();
        }catch(Exception ex){
            throw new multex.Failure("Failure deleting the application "
                + "specific database tables", ex
            );
        }

    }

    /** Mark the beginning of an transaction.
      Unconfirms all modifications made to the database after the last
      call to {@link #endTransaction(boolean)}.
      Note: This operation is conceptually static.
      It does not make use of the attribute values of the actual (dummy) object.
     */
    public static void dbBeginTransaction()
    { _dbApplication.getSession().rollback(); }

    /** Confirm or unconfirm the modifications made to the database in the
      actual transaction (since the last commit) and close this transaction.
      Note: This operation is conceptually static.
      It does not make use of the attribute values of the actual (dummy) object.
      @param i_ok true if the transaction succeeded, then the modifications
        will be confirmed, otherwise unconfirmed.
     */
    public static void dbEndTransaction(final boolean i_ok)
    {
        final jora.Session session = _dbApplication.getSession();
        if(i_ok){session.commit();}else{session.rollback();}
    }

    private static DbApplication _dbApplication;

}