/*
 * LgKonto.java
 *
 * 2002-04-29  Knabe  Aufgetrennt in Interface LgKonto und seine LgKontoImpl
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:44
 */

package bank3tier.lg;
import  bank3tier.db.DbObject;

/**
 * Ein Konto bei einer Bank.
 * @author  Christoph Knabe
 */
public interface LgKonto extends LgObject {

    /**Returns the actual account balance in cents*/
    public int getSaldo();

    /**The minimal account balance, which the bearer of the account may not
      subcede.
    */
    public static final int mindestSaldo = -1000000;

    /**Adds the client i_kunde as an additional manager of this account.*/
    public void verwalterHinzufgen(final LgKunde i_kunde)throws SchonVerwalterExc;

    /**Sends the money amount i_betrag in cents from this account to the
      destination account io_zielKonto. The operation is only allowed, if the
      sender client i_auftraggeben is an authorized manager of this source
      account.
    */
    public void berweisen(
        final LgKonto io_zielKonto, final int i_betrag,
        final LgKunde i_auftraggeber
    ) throws MindestSaldoExc, OhneRechteExc;

    public class OhneRechteExc extends multex.Exc {
        public OhneRechteExc(final LgKunde i_kunde, final LgKonto i_konto){
            super("Kunde {0} darf nicht ber Konto mit OID {1} verfgen",
                i_kunde.getName(), i_konto.getOid()
            );
        }
    }

    public class MindestSaldoExc extends multex.Exc {
        public MindestSaldoExc(final int i_mindestSaldo){
            super("Mindestsaldo {0} Cent wrde unterschritten werden",
                new Long(i_mindestSaldo)
            );
        }
    }

    public class SchonVerwalterExc extends multex.Exc {
        public SchonVerwalterExc(final LgKunde i_kunde, final LgKonto i_konto){
            super("Kunde {0} ist schon Verwalter von Konto mit OID {1}",
                i_kunde.getName(), i_konto.getOid()
            );
        }
    }

}