/*
 * LgKonto.java
 *
 * 2002-04-29  Knabe  Aufgetrennt in Interface LgKonto und seine LgKontoImpl
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:44
 */

package bank3tier.lg;
import  bank3tier.db.DbObject;
import  bank3tier.db.DbObjectImpl;
import  bank3tier.db.DbService;

/**
 * Ein Konto bei einer Bank.
 * @author  Christoph Knabe
 */
/*package*/ class LgKontoImpl extends DbObjectImpl implements LgKonto {
    //Attribute:
    private int saldo = 0;
    private java.sql.Date eroeffnet
    = new java.sql.Date(new java.util.Date().getTime());
    private java.sql.Date geschlossen = null;

    public int getSaldo(){return saldo;}

    /* Returns the LgKonto object with the OID i_oid.
     *  @throws NotFoundExc If in i_table no object with OID i_oid was found.
     *  @see bank3tier.db.DbObject

    public static LgKonto selectByOid(final Integer i_oid)
    throws OidNotFoundExc
    {
        return (LgKonto)dbSelectByOid(table, i_oid);
    }
    */

    public void verwalterHinzufgen(final LgKunde i_kunde)throws LgKonto.SchonVerwalterExc{
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            final LgKundeImpl kunde = (LgKundeImpl)i_kunde;
            this.dbRetrieve();   kunde.dbRetrieve();
            if(LgVerwaltetImpl.dummy.dbAreLinked(kunde, this))
            {throw new SchonVerwalterExc(i_kunde, this);}

            final LgVerwaltetImpl verwaltet = new LgVerwaltetImpl(i_kunde, this);
            verwaltet.setzen(false); //kein Inhaber
            verwaltet.dbInsert();
            ok = true;
        }catch(LgKonto.SchonVerwalterExc ex){throw ex;
        }catch(Exception ex){
            throw new multex.Failure(
                "Verwalter konnte nicht hinzugefgt werden.", ex
            );
        }
        finally{DbService.dbEndTransaction(ok);}
    }

    public void berweisen(
        final LgKonto io_zielKonto, final int i_betrag,
        final LgKunde i_auftraggeber
    ) throws LgKonto.MindestSaldoExc, LgKonto.OhneRechteExc
    {
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            final LgKontoImpl zielKonto = (LgKontoImpl)io_zielKonto;
            final LgKundeImpl auftraggeber = (LgKundeImpl)i_auftraggeber;
            //1. Arbeitsspeicher-Objekte aus DB aktualisieren:
            this.dbRetrieve();   zielKonto.dbRetrieve();
            auftraggeber.dbRetrieve();
            //2. Fehlerprfung:
            final int neuSaldo = saldo - i_betrag;
            if(neuSaldo < mindestSaldo){throw new LgKonto.MindestSaldoExc(mindestSaldo);}
            if(!LgVerwaltetImpl.dummy.dbAreLinked(auftraggeber, this))
            {throw new LgKonto.OhneRechteExc(i_auftraggeber, this);}
            //3. Vernderungen im Arbeitsspeicher vornehmen:
            this.saldo = neuSaldo;
            ((LgKontoImpl)io_zielKonto).saldo += i_betrag;
            //4. Vernderungen in DB sichern:
            this.dbUpdate();
            zielKonto.dbUpdate();
            ok = true;
        }catch(LgKonto.MindestSaldoExc ex){throw ex;
        }catch(LgKonto.OhneRechteExc ex){throw ex;
        }catch(Exception ex){throw new multex.Failure(
            "Betrag {0} konnte nicht berwiesen werden", ex, new Long(i_betrag)
            );
        }finally{DbService.dbEndTransaction(ok);}
    }

    private static final jora.Table table = DbService.dbNewTable(LgKontoImpl.class);
    public jora.Table dbTable(){return table;}

}//LgKontoImpl