package bank3tier.lg; //LgKunde.java
import  bank3tier.db.DbObject;
import  bank3tier.db.DbObjectImpl;
import  bank3tier.db.DbService;

/*
 * LgKunde.java
 *
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:34
 */

/**
 * @author  Christoph Knabe
 */
public class LgKundeImpl extends DbObjectImpl implements LgKunde {
    private String name;
    private java.sql.Date geburt;
    //private LgDatum geburt; //geht nicht, obwohl von java.sql.Date abgeleitet

    public void erfassen(){
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            dbInsert();
            ok = true;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu erfassen", ex, name);
        }finally{DbService.dbEndTransaction(ok);}
    }

    /**ndert das Kundenobjekt als externe, geprfte Transaction*/
    public void aendern(final String i_name, final LgDatum i_geburt){
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            dbRetrieve();
            setzen(i_name, i_geburt);
            dbUpdate();
            ok = true;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu ndern", ex, i_name);
        }finally{DbService.dbEndTransaction(ok);}
    }

    /**Lscht das Kunde object as external checked transaction*/
    public void loeschen(){
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            dbRetrieve();
            dbDelete();
            ok = true;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu lschen", ex, getName());
        }finally{DbService.dbEndTransaction(ok);}
    }

    public LgKonto kontoErffnen(){
        //throw new RuntimeException("Noch nicht implementiert");
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            final LgKontoImpl konto = new LgKontoImpl();
            konto.dbInsert();
            final LgVerwaltetImpl verwaltet = new LgVerwaltetImpl(this, konto);
            verwaltet.setzen(true); //als Inhaber
            verwaltet.dbInsert();
            ok = true;
            return konto;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, ein Konto zu erffnen", ex);
        }finally{DbService.dbEndTransaction(ok);}
/*
*/
    }

    /**Sets these attributes in the actual object.
      As internal, unpersistent operation
    */
    public void setzen(final String i_name, final LgDatum i_geburt){
        name = i_name;   geburt = i_geburt;
    }

    public String getName(){return name;}
    public void setName(String i_name){name = i_name;}
    public LgDatum getGeburt(){return new LgDatum(geburt);}
    public void setGeburt(final LgDatum i_geburt){geburt = i_geburt;}


    public LgKunde_List alleSuchen(){
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            final jora.Cursor c = table.select("");
            final LgKunde_List result = new LgKunde_List();
            result.addAll(c);
            return result;
        }catch(Exception ex){throw new multex.Failure(
            "Failure searching all Kunde-objects", ex);
        }finally{DbService.dbEndTransaction(ok);}
    }

    public LgKunde suchen(final LgDatum i_geburtsDatum) //static
    throws KeinKundeExc
    {
        DbService.dbBeginTransaction();
        boolean ok = false;
        try {
            final LgKundeImpl example = new LgKundeImpl();
            example.setGeburt(i_geburtsDatum);
            final LgKunde result = (LgKunde)example.dbSelectUniqueByExample();
            ok = true;
            return result;
        }catch(DbObject.ExampleNotFoundExc ex){throw new KeinKundeExc(i_geburtsDatum);
        }catch(Exception ex){throw new multex.Failure(
            "Fehler beim Suchen eines Kunden", ex, i_geburtsDatum);
        }finally{DbService.dbEndTransaction(ok);}
    }

    public LgKontoStand_List getKontoStandList(){
        DbService.dbBeginTransaction();
        boolean ok = false;
        try{
            //QueryByExample: Alle Verwaltet-Links dieses Kunden:
            final LgVerwaltetImpl query = new LgVerwaltetImpl(this);
            final jora.Cursor c = query.table.queryByExample(query);
            final LgKontoStand_List result = new LgKontoStand_List();
            for(;;){
                final LgVerwaltetImpl verwaltet = (LgVerwaltetImpl)c.next();
                if(verwaltet==null){break;}
                final LgKonto konto = (LgKonto)verwaltet.dbGetlink(LgKontoImpl.class);
                final LgKontoStand kontoStand = new LgKontoStand(
                    konto.getOid(), verwaltet.inhaber(), konto.getSaldo()
                );
                result.add(kontoStand);
            }
            c.close();
            return result;
        }catch(Exception ex){throw new multex.Failure(
            "Fehler beim Ermitteln der KontenStnde for Kunde OID {0}", ex, getOid());
        }finally{DbService.dbEndTransaction(ok);}
    }

    public String toString(){
        return dbTable().name() + " {name=" + name + ", geburt=" + geburt + "}";
    }

    /** Creates new empty LgKunde */
    /*package*/ LgKundeImpl() {}

    private static final jora.Table table = DbService.dbNewTable(LgKundeImpl.class);
    public jora.Table dbTable(){return table;}

}//LgKundeImpl

