package joraTest; //OidTest_int.java

//Purpose: Test program for usage of JORA with database objects, that are
//uniquely identified by an Object IDentification (oid).

/*With OID type int and Access97 it seems to work.
*/

//2001-05-03  Knabe  Type of oid changed to int.
//2001-05-03  Knabe  Now working. New Method DbObject.selectOid(Integer)
//2001-04-20  Knabe  Type of oid changed to Integer
//2001-04-30  Knabe:  Moved to package joraTest, type-safe Table-constructors,
//                    without import-s.
//2001-03-09  Knabe  Control compilation and test
//2001-02-20  Knabe  Added m:n-association Employee <--> Project as WorkingOn
//2001-02-19  Knabe  With private/protected attributes, base class DbObject
//2001-02-13  Knabe  Adaptation to MulTEx und Access97
//Jora2.06  Creation as file Test.java


/**Subclass of jora.Table with more type-safe constructors, to which you have to
  give a Class-object rather than a string naming the class of the objects to be
  stored. E.g.: <PRE>
  *  public static jora.Table table = new Table(Employee.class, null,Test.session,"code");
  </PRE>
*/
class Table extends jora.Table {
    /**Type-safe constructor for table object. Makes association between Java class
     *  i_class and database table i_tableName.
     *
     * @i_key key table's primary key. This parameter is used in UPDATE/DELETE
     *  operations to locate record in the table.
     * @io_s session, which should be opened before first access to the table
     */
    public Table(Class i_class, String i_tableName, jora.Session io_s, String i_key) {
        super(i_class.getName(), i_tableName, io_s, i_key);
    }

    /**Constructor of table with "key" and "session" parameters inherited
     *  from base table.
     */
    public Table(Class i_class) {
        super(i_class.getName());
    }

   /**Constructor of table with "tableName" derived from name of i_class
   */
    public Table(Class i_class, jora.Session io_s, String i_key) {
        super(i_class.getName(), io_s, i_key);
    }

}

/**redefines the method handleSqlException in order to give more detailed
  exception cause and location information to the upper tiers
*/
class MySession extends jora.Session {
    public void handleSQLException(java.sql.SQLException ex) throws jora.Failure {
        throw new jora.Failure(ex);
    }
    public static jora.Session session(){return _session;}
    private static jora.Session _session = new MySession();
}

abstract class DbObject {
    public final int oid(){return oid;}

    /** Insert a new record into the table. Values of inserted record fields
     *  are taken from the actual object.
    */
    public final void insert() {
        oid = _getNewOid();
        table().insert(this);
    }

    /** Returns the object with the OID i_oid from the database table i_table.
     *  The returned object will be dynamically an instance of the type given 
     *  as first parameter to the constructor of i_table.
     *  Note: The selected object should
     *  have now components of InputStream, Blob or Clob type, because
     *  their data will be not available after closing the Cursor used to get 
     *  it.
     *  
     *  @throws ObjectNotFoundExc If in i_table no object with OID i_oid was found.
     */
    public static Object selectOid(final jora.Table i_table, final int i_oid) 
    throws ObjectNotFoundExc
    {
        final jora.Cursor c = i_table.selectAll("where oid="+i_oid);
        final Object result = c.next();
        if(result==null){
            throw new ObjectNotFoundExc(i_oid, i_table);
        }
        //?Wegen Verlust der BLOBS usw. so sinnvoll?
        c.close();
        return result;
    }

    /** Update record in the table using table's primary key to locate
     *  record in the table and values of fields from the actual object
     *  to alter the record fields.
     * @return number of objects actually updated
    */
    public final int update() {
        return table().update(this);
    }

    /** Delete record with specified value of primary key from the table.
     *  The actual object is taken to determine the value of the primary key.
     * @return number of objects actually deleted
     */
    public final int delete() {
        return table().delete(this);
    }
    
    public static class ObjectNotFoundExc extends multex.Exc {
        public ObjectNotFoundExc(final int i_oid, final jora.Table i_table){
            super("Object with OID {0} not found in table \"{1}\"", 
                new Integer(i_oid), i_table.name()
            );
        }
    }

    /** Get the table responsible for storing objects of the same class as this.
      * It is necessary to redefine this method in each concrete class by <PRE>
      *   jora.Table table(){return table;}
      * </PRE>
    */
    protected abstract jora.Table table();

    /**An object identification number unique for each object in our application
    */
    private int        oid = 0; //0 means "not yet stored"

    private static int _lastOid = 0;

    /** Returns an object identification number unique for each object in our
      * application. in a non-demo application with real persistency the
      * _lastOid should be stored in the database, too, rather than in a
      * variable.
    */
    private static int _getNewOid(){return ++_lastOid;}
}

class Employee extends DbObject {
    public void set(
        final String i_name, final int i_age,
        final java.sql.Date i_hire, final double i_salary
    ){ name=i_name; age=i_age; hire=i_hire; salary=i_salary; }
    private String     name;
    public  String     name(){return name;}
    public void setName(final String i_name){name=i_name;}

    public  int        age(){return age;}
    private int        age;

    public  java.sql.Date hire(){return hire;}
    private java.sql.Date hire;

    public  double     salary(){return salary;}
    private double     salary;

    public String toString() {
        return "#" + oid() + '\t' + name + '\t' + age + '\t' + hire + '\t'
          + salary;
    }
    /*package*/ static jora.Table table
    = new Table(Employee.class, null, MySession.session(), "oid");
    protected jora.Table table(){return table;}
}


class Manager extends Employee {
    public void set(final int i_projectId, final Experience i_experience)
    {projectId = i_projectId;   experience = i_experience;}

    public String toString() {
        String s = super.toString() + '\t' + projectId;
        for (Experience exp=experience; exp != null; exp = exp.next) {
            s += '\t' + exp.text;
        }
        return s;
    }

    /**manages the project with this projectId*/
    int        projectId;
    Experience experience;
    private static jora.Table table = new Table(Manager.class);
    protected jora.Table table(){return table;}
}


class Programmer extends Employee {
    public String     language;
    public Activity   project;
    public int        skill;

    public String toString() {
        return super.toString() + '\t' + language + '\t' + skill;
    }
    /*package*/ static jora.Table table = new Table(Programmer.class);
    protected jora.Table table(){return table;}
}


class Experience implements java.io.Serializable {
    public Experience next;
    public String     text;

    public Experience(String txt, Experience chain) {
        next = chain;
	text = txt;
    }
}


class Project extends DbObject {
    private String title;
    public  String getTitle(){return title;}
    public  void   setTitle(final String i_title){title = i_title;}

    /*package*/ static jora.Table table
    = new Table(Project.class, MySession.session(), "oid");
    protected jora.Table table(){return table;}
}


/** A m:n association between Employee and Project.
  An employee is working on n projects. A project has m employees working in it.
*/
class WorkingOn extends DbObject {
    private int employeeOid;
    private int projectOid;

    public WorkingOn(final Employee i_employee, final Project i_project){
        employeeOid = i_employee.oid();   projectOid = i_project.oid();
    }
    public WorkingOn(){}

    public int employeeOid(){return employeeOid;}
    public int projectOid (){return projectOid;}

    /*package*/ static jora.Table table = new Table(WorkingOn.class, MySession.session(),"oid");
    protected jora.Table table(){return table;}
}


class Activity {
    public int    projectId;
    public float  percentOfTime;
    public String role;

    public Activity(int id, float percent, String position) {
        projectId = id;
	percentOfTime = percent;
	role = position;
    }
    public Activity() {}
}

/**Purpose: Test program for usage of JORA with database objects, that are
  uniquely identified by an Object IDentification (oid) of type int.
  @author Christoph Knabe {@link //www.tfh-berlin.de/~knabe/} of OidTest.java and the MulTEx package
  @author Konstantin Knizhnik {@link //www.ispras.ru/~knizhnik/} of Test.java and the JORA package

*/
class OidTest_int {

    public static final String dataSource = "jdbc:odbc:JoraTest";
    public static final String user = "";
    public static final String password = "";

    public static void main(String[] args){
      try{_performTest();}
      catch(Exception ex){multex.Msg.printReport(ex);}
    }

    private static void _performTest() throws Exception
    {
        Employee    emp;
	Programmer  p;
	jora.Cursor c;
	int         i;

        final jora.Session session = MySession.session();

        session.open(dataSource, user, password);

	session.execute("create table Manager ("
            +"oid INTEGER CONSTRAINT ManagerKey PRIMARY KEY,"
            +"name VARCHAR(50) NOT NULL,"
            +"age INTEGER,"
            +"hire DATETIME,"
            +"salary NUMERIC,"
            +"projectId INTEGER,"
            +"experience IMAGE)"
        );

	session.execute("create table Programmer ("
            +"oid INTEGER CONSTRAINT ProgrammerKey PRIMARY KEY,"
            +"name VARCHAR(50) NOT NULL,"
            +"age INTEGER,"
            +"hire DATETIME,"
            +"salary NUMERIC,"
            +"language CHAR(8),"
            +"project_projectId INTEGER,"
            +"project_percentOfTime REAL,"
            +"project_role VARCHAR(100),"
            +"skill SMALLINT)"
        );

        session.execute("create table Project ( "
            +"oid INTEGER CONSTRAINT ProjectKey PRIMARY KEY"
            +", title VARCHAR(50) NOT NULL"
            +" )"
        );

        session.execute("create table WorkingOn ( "
            +"oid INTEGER CONSTRAINT WorkingOnKey PRIMARY KEY"
            +", employeeOid INTEGER"
            +", projectOid  INTEGER"
            +" )"
        );

        final Programmer jons = new Programmer();
        jons.set("Jons", 34, java.sql.Date.valueOf("1990-5-1"), 80000.00);
        jons.language = "C++";
        jons.project = new Activity(123, 0.5f, "coder");
        jons.skill = 5;

        //Programmer.table.insert(jons);
        jons.insert();

        final Programmer smith = new Programmer();
        smith.set("Smith", 25, java.sql.Date.valueOf("1995-1-1"), 100000.00);
        smith.language = "C++";
        smith.project = new Activity(123, 1.0f, "tester");
        smith.skill = 3;

        //Programmer.table.insert(smith);
        smith.insert();

        final Manager boers = new Manager();
        boers.set("Boers", 43, java.sql.Date.valueOf("1997-1-1"), 220000.99);
        boers.projectId = 123;
        boers.experience = new Experience("DECbank", boers.experience);
        boers.experience = new Experience("Teleform", boers.experience);

        //Manager.table.insert(boers);
        boers.insert();

        smith.language = "Java";
        smith.update();

        //Create some projects and assign employees working on them:
        {
            final Project project = new Project();
            project.setTitle("Kienzle");
            project.insert();
            new WorkingOn(boers, project).insert();
            new WorkingOn(jons,  project).insert();
            new WorkingOn(smith, project).insert();
        }
        {
            final Project project = new Project();
            project.setTitle("Flohr-Otis");
            project.insert();
            new WorkingOn(boers, project).insert();
            new WorkingOn(jons,  project).insert();
        }
        {
            final Project project = new Project();
            project.setTitle("Gildemeister");
            project.insert();
            new WorkingOn(boers, project).insert();
        }


        Object[] ra = Employee.table.selectAll("").toArray(100);
        System.out.println("All employees");
        System.out.println("------------------------");
        for (i = 0; i < ra.length; i++) {
          emp = (Employee)ra[i];
          System.out.println(emp.toString());
        }
        System.out.println("------------------------\n\n");
        System.out.println("Well paid employees");
        System.out.println("------------------------");
        ra = Employee.table.selectAll("where salary > 100000").toArray();
        for (i = 0; i < ra.length; i++) {
          emp = (Employee)ra[i];
          System.out.println(emp.name() + "\t" + emp.salary());
        }

        {
            System.out.println
            ("----------------------------------------------------\n\n");
            System.out.println
            ("List all projects with the employees working on them");
            System.out.println
            ("----------------------------------------------------");
            for(final jora.Cursor projectCursor = Project.table.select("");;){
                final Project project = (Project)projectCursor.next();
                if(project==null){break;}
                System.out.print(project.getTitle());
                //Print the employees of the project:
                final jora.Cursor woCursor
                = WorkingOn.table.select("where projectOid = " + project.oid());
                for(;;){
                    final WorkingOn wo = (WorkingOn)woCursor.next();
                    if(wo==null){break;}
                    final int employeeOid = wo.employeeOid();
                    System.out.print("\t#" + employeeOid + "=");
                    final Employee employee
                    = (Employee)Employee.selectOid(Employee.table, employeeOid);
                    System.out.print(employee.name());
                }
                System.out.println();
            }
            System.out.println
            ("------------------------------------------------\n\n");
        }

        // Discharge all testers:
        //c = Programmer.table.select("where project_role='tester' for update");
        //FOR UPDATE does not work with ACCESS: Knabe
        c = Programmer.table.select("where project_role='tester'");
        while ((p = (Programmer)c.next()) != null) {
            //c.delete(); //does not work with ACCESS
            //Programmer.table.delete(p);  //so it works with ACCESS. Knabe
            p.delete();  //so still shorter. Knabe
        }
        // ...and let other programmers do their job
        //c = Programmer.table.select("for update");
        c = Programmer.table.select("");
        while ((p = (Programmer)c.next()) != null) {
            if (p.project.percentOfTime < 0.8f) {
                p.project.percentOfTime = 0.8f;
                //c.update();
                p.update();
          }
        }

        boers.delete(); // Reduce number of managers


        System.out.println("------------------------\n\n");
        System.out.println("Employees with name Jons");
        System.out.println("------------------------");
        emp = new Employee();
        emp.setName("Jons");
        c = Employee.table.queryAllByExample(emp);
        while ((emp = (Employee)c.next()) != null) {
          System.out.println("#" + emp.oid() + " " + emp.name());
        }
        System.out.println("------------------------");
        session.closePreparedStatements(); //Knabe 2001-02-16
        session.execute("drop table Manager");
        session.execute("drop table Programmer");
        session.execute("drop table Project");
        session.execute("drop table WorkingOn");

        session.close();
    }//_performTest
}//OidTest_int
