package joraTest;  //Test.java

//2001-04-30  Knabe:  Moved to package joraTest, type-safe Table-constructors,
//                    without import-s.
//2001-02-13  Christoph Knabe: Adaptation to MulTEx and Access
//Jora2.06  Knizhnik: Creation

/**redefines the method handleSqlException in order to give more detailed
  exception cause and location information to the upper tiers
*/
class MultexSession extends jora.Session {
  public void handleSQLException(java.sql.SQLException ex) throws jora.Failure {
    throw new jora.Failure(ex,null);
  }/*
  public void handleSQLException(SQLException ex, String sql) throws jora.Failure {
    throw new jora.Failure(ex,sql);
  }*/
}


/**Subclass of jora.Table with more type-safe constructors, to which you have to
  give a Class-object rather than a string naming the class of the objects to be
  stored. E.g.: <PRE>
  *  public static jora.Table table = new Table(Employee.class, null,Test.session,"code");
  </PRE>
*/
class Table extends jora.Table {
    /**Type-safe constructor for table object. Makes association between Java class
     *  i_class and database table i_tableName.
     *
     * @i_key Table's primary key. This parameter is used in UPDATE/DELETE
     *  operations to locate record in the table.
     * @io_s Session, which should be opened before first access to the table
     */
    public Table(Class i_class, String i_tableName, jora.Session io_s, String i_key) {
        super(i_class.getName(), i_tableName, io_s, i_key);
    }

    /**Constructor of table with "key" and "session" parameters inherited
     *  from base table.
     */
    public Table(Class i_class) {
        super(i_class.getName());
    }

}

class Employee {
    public int        code;
    public String     name;
    public int        age;
    public java.sql.Date       hire;
    public double     salary;

    public static Table table = new Table(Employee.class, null,Test.session,"code");
    public String toString() {
        return "#" + code + '\t' + name + '\t' + age + '\t' + hire + '\t'
	  + salary;
    }
}


class Manager extends Employee {
    public int        projectId;
    public Experience experience;

    public String toString() {
        String s = super.toString() + '\t' + projectId;
	for (Experience exp=experience; exp != null; exp = exp.next) {
	    s += '\t' + exp.text;
	}
	return s;
    }
    public static Table table = new Table(Manager.class);
}


class Experience implements java.io.Serializable {
    public Experience next;
    public String     text;

    public Experience(String txt, Experience chain) {
        next = chain;
	text = txt;
    }
}


class Programmer extends Employee {
    public String     language;
    public Activity   project;
    public int        skill;

    public String toString() {
        return super.toString() + '\t' + language + '\t' + skill;
    }
    public static Table table = new Table(Programmer.class);
}


class Activity {
    public int    projectId;
    public float  percentOfTime;
    public String role;

    public Activity(int id, float percent, String position) {
        projectId = id;
	percentOfTime = percent;
	role = position;
    }
    public Activity() {}
}


class Test {
    public static jora.Session session; // = new MultexSession();

    public static final String dataSource = "jdbc:odbc:JoraTest";
    public static final String user = "";
    public static final String password = "";

    public static void main(String[] args){
      try{_performTest();}
      catch(Throwable ex){ multex.Msg.printReport(ex);
         //ex.printStackTrace();
      }
    }

    private static void _performTest() throws java.sql.SQLException
    {
	Programmer p;
	Manager    m;
	jora.Cursor     c;
	int        i;
        /*
      session.open(dataSource, user, password);


	session.execute("create table Manager ("
      //+"code INTEGER PRIMARY KEY,"
      +"code INTEGER CONSTRAINT ManagerKey PRIMARY KEY,"
			+"name VARCHAR(50) NOT NULL,"
			+"age INTEGER,"
			+"hire DATETIME,"
			+"salary NUMERIC,"
			+"projectId INTEGER,"
			+"experience IMAGE)");

	session.execute("create table Programmer ("
      //+"code INTEGER PRIMARY KEY,"
      +"code INTEGER CONSTRAINT ProgrammerKey PRIMARY KEY,"
			+"name VARCHAR(50) NOT NULL,"
			+"age INTEGER,"
			+"hire DATETIME,"
			+"salary NUMERIC,"
			+"language CHAR(8),"
			+"project_projectId INTEGER,"
			+"project_percentOfTime REAL,"
			+"project_role VARCHAR(100),"
			+"skill SMALLINT)");
/**/
      p = new Programmer();
      p.code = 100;
      p.name = "Jons";
      p.age = 34;
      p.hire = java.sql.Date.valueOf("1990-5-1");
      p.salary = 80000.00;
      p.language = "C++";
      p.project = new Activity(123, 0.5f, "coder");
      p.skill = 5;
      Programmer.table.insert(p);

      p = new Programmer();
      p.code = 101;
      p.name = "Smith";
      p.age = 25;
      p.hire = java.sql.Date.valueOf("1995-1-1");
      p.salary = 100000.00;
      p.language = "C++";
      p.project = new Activity(123, 1.0f, "tester");
      p.skill = 3;

      Programmer.table.insert(p);

      m = new Manager();
      m.code = 201;
      m.name = "Boers";
      m.age = 43;
      m.hire = java.sql.Date.valueOf("1997-1-1");
      m.salary = 220000.99;
      m.projectId = 123;
      m.experience = new Experience("DECbank", m.experience);
      m.experience = new Experience("Teleform", m.experience);


      Manager.table.insert(m);

      p.language = "Java";
      Programmer.table.update(p);

      Object[] ra = Employee.table.selectAll("").toArray(100);
      System.out.println("All employees");
      System.out.println("------------------------");
      for (i = 0; i < ra.length; i++) {
          final Employee emp = (Employee)ra[i];
          System.out.println(emp.toString());
      }
      System.out.println("------------------------\n\n");
      System.out.println("Well paid employess");
      System.out.println("------------------------");
      ra = Employee.table.selectAll("where salary > 100000").toArray();
      for (i = 0; i < ra.length; i++) {
          final Employee emp = (Employee)ra[i];
          System.out.println(emp.name + "\t" + emp.salary);
      }

      // Discharge all testers
      //c = Programmer.table.select("where project_role='tester' for update");  //FOR UPDATE Does not work with ACCESS!!
      c = Programmer.table.select("where project_role='tester'");
      while ((p = (Programmer)c.next()) != null) {
            //c.delete();
            Programmer.table.delete(p); //so works with ACCESS
      }
      // ...and let other programmers do their job
      //c = Programmer.table.select("for update");
      c = Programmer.table.select("");
      while ((p = (Programmer)c.next()) != null) {
          if (p.project.percentOfTime < 0.8f) {
              p.project.percentOfTime = 0.8f;
                //c.update();
                Programmer.table.update(p);
          }
      }

      Manager.table.delete(m); // Reduce number of managers


      System.out.println("------------------------\n\n");
      System.out.println("Employees with name Jons");
      System.out.println("------------------------");
      Employee emp = new Employee();
      emp.name = "Jons";
      c = Employee.table.queryAllByExample(emp);
      while ((emp = (Employee)c.next()) != null) {
          System.out.println("#" + emp.code + " " + emp.name);
      }
      System.out.println("------------------------");
        session.closePreparedStatements(); //Knabe 01-02-16
	session.execute("drop table Manager");
	session.execute("drop table Programmer");

	session.close();
    }
}
